@@ -156,6 +156,7 @@ class SkypeConnection(SkypeObj):
         self.tokens = {}
         self.tokenExpiry = {}
         self.tokenFile = None
+        self.hasUserPwd = False
         self.msgsHost = self.API_MSGSHOST
         self.sess = requests.Session()
         self.endpoints = {"self": SkypeEndpoint(self, "SELF")}
@@ -170,6 +171,23 @@ class SkypeConnection(SkypeObj):
     def guest(self):
         return self.userId.startswith("guest:") if self.userId else None
 
+    def closure(self, method, *args, **kwargs):
+        """
+        Create a generic closure to call a method with fixed arguments.
+
+        Args:
+            method (MethodType): bound method of the class
+            args (list): positional arguments for the method
+            kwargs (dict): keyword arguments for the method
+
+        Returns:
+            MethodType: bound method closure
+        """
+        @functools.wraps(method)
+        def inner(self):
+            return method(*args, **kwargs)
+        return MethodType(inner, self)
+
     def __call__(self, method, url, codes=(200, 201, 202, 204, 207), auth=None, headers=None, **kwargs):
         """
         Make an API call.  Most parameters are passed directly to :mod:`requests`.
@@ -253,19 +271,6 @@ class SkypeConnection(SkypeObj):
                 states.append(state)
         return resp
 
-    def setUserPwd(self, user, pwd):
-        """
-        Replace the stub :meth:`getSkypeToken` method with one that connects via the Microsoft account flow using the
-        given credentials.  Avoids storing the account password in an accessible way.
-
-        Args:
-            user (str): username or email address of the connecting account
-            pwd (str): password of the connecting account
-        """
-        def getSkypeToken(self):
-            self.liveLogin(user, pwd)
-        self.getSkypeToken = MethodType(getSkypeToken, self)
-
     def setTokenFile(self, path):
         """
         Enable reading and writing session tokens to a file at the given location.
@@ -345,6 +350,26 @@ class SkypeConnection(SkypeObj):
             if "reg" not in self.tokenExpiry or datetime.now() >= self.tokenExpiry["reg"]:
                 self.getRegToken()
 
+    def skypeTokenClosure(self, method, *args, **kwargs):
+        """
+        Replace the stub :meth:`getSkypeToken` method with one that connects using the given credentials.  Avoids
+        storing the account password in an accessible way.
+        """
+        self.getSkypeToken = self.closure(method, *args, **kwargs)
+        self.hasUserPwd = True
+
+    def setUserPwd(self, user, pwd):
+        """
+        Replace the stub :meth:`getSkypeToken` method with one that connects via SOAP login using the given
+        credentials.  Avoids storing the account password in an accessible way.
+
+        Args:
+            user (str): username or email address of the connecting account
+            pwd (str): password of the connecting account
+        """
+        login = self.soapLogin if "@" in user else self.liveLogin
+        self.skypeTokenClosure(login, user, pwd)
+
     def liveLogin(self, user, pwd):
         """
         Obtain connection parameters from the Microsoft account login page, and perform a login with the given email
@@ -365,6 +390,35 @@ class SkypeConnection(SkypeObj):
             .SkypeAuthException: if the login request is rejected
             .SkypeApiException: if the login form can't be processed
         """
+        if not self.hasUserPwd:
+            self.skypeTokenClosure(self.liveLogin, user, pwd)
+        self.tokens["skype"], self.tokenExpiry["skype"] = SkypeLiveAuthProvider(self).auth(user, pwd)
+        self.getUserId()
+        self.getRegToken()
+
+    def soapLogin(self, user, pwd):
+        """
+        Perform a login with the given email address or Skype username, and its password, using the Microsoft account
+        SOAP login APIs.
+
+        .. note::
+            Microsoft accounts with two-factor authentication enabled are supported if an application-specific password
+            is provided.  Skype accounts must be linked to a Microsoft account with an email address, otherwise
+            :class:`.SkypeAuthException` will be raised.  See the exception definitions for other possible causes.
+
+        Args:
+            user (str): username or email address of the connecting account
+            pwd (str): password of the connecting account
+
+        Returns:
+            (str, datetime.datetime) tuple: Skype token, and associated expiry if known
+
+        Raises:
+            .SkypeAuthException: if the login request is rejected
+            .SkypeApiException: if the login form can't be processed
+        """
+        if not self.hasUserPwd:
+            self.skypeTokenClosure(self.soapLogin, user, pwd)
         self.tokens["skype"], self.tokenExpiry["skype"] = SkypeSOAPAuthProvider(self).auth(user, pwd)
         self.getUserId()
         self.getRegToken()
@@ -390,7 +444,7 @@ class SkypeConnection(SkypeObj):
 
     def getSkypeToken(self):
         """
-        A wrapper for :meth:`liveLogin` that applies the previously given username and password.
+        A wrapper for the default login provider that applies the previously given username and password.
 
         Raises:
             .SkypeAuthException: if credentials were never provided
@@ -499,6 +553,16 @@ class SkypeAPIAuthProvider(SkypeAuthProvider):
         return json["skypetoken"], expiry
 
 
+class LiveAuthSuccess(Exception):
+    """
+    An exception used to capture the 't' value needed during Microsoft account authentication.
+    """
+
+    def __init__(self, t):
+        super(LiveAuthSuccess, self).__init__(t)
+        self.t = t
+
+
 class SkypeLiveAuthProvider(SkypeAuthProvider):
     """
     An authentication provider that connects via Microsoft account authentication.
@@ -537,43 +601,67 @@ class SkypeLiveAuthProvider(SkypeAuthProvider):
             .SkypeAuthException: if the login request is rejected
             .SkypeApiException: if the login form can't be processed
         """
-        # Do the authentication dance.
-        params = self.getParams()
-        t = self.sendCreds(user, pwd, params)
-        return self.getToken(t)
+        try:
+            self.getT(user, pwd)
+        except LiveAuthSuccess as ex:
+            return self.getToken(ex.t)
 
-    def getParams(self):
-        # First, start a Microsoft account login from Skype, which will redirect to login.live.com.
-        loginResp = self.conn("GET", "{0}/oauth/microsoft".format(SkypeConnection.API_LOGIN),
-                              params={"client_id": "578134", "redirect_uri": "https://web.skype.com"})
+    def check(self, resp):
+        page = BeautifulSoup(resp.text, "html.parser")
+        # Look for the 't' value we need to exchange for a Skype token, which might turn up at any stage.
+        tField = page.find(id="t")
+        if tField is not None:
+            raise LiveAuthSuccess(tField.get("value"))
+        # Look for an error message within the response.
+        errReg = re.search(r"sErrTxt:'([^'\\]*(\\.[^'\\]*)+)'", resp.text)
+        if errReg:
+            errMsg = re.sub(r"<.*?>", "", errReg.group(1)).replace("\\'", "'").replace("\\\\", "\\")
+            raise SkypeApiException(errMsg, resp)
+        # Look for two-factor authentication device information (a non-empty array of factors) that we can't handle.
+        if re.search(r"\bV:\s*\[\s*{", resp.text):
+            raise SkypeAuthException("Two-factor authentication unsupported", resp)
+        # Look for a user consent form, meaning the user needs to accept terms or follow account security steps.
+        for form in page.findAll("form"):
+            if form["name"] == "fmHF":
+                url = form["action"].split("?", 1)[0]
+                raise SkypeAuthException("Account action required ({0}), login with a web browser first"
+                                         .format(url), resp)
+        # No common elements, return the response for further processing.
+        return resp
+
+    def getT(self, user, pwd):
+        # Stage 1: Start a Microsoft account login from Skype, which will redirect to login.live.com.
+        stage1Resp = self.check(self.conn("GET", "{0}/oauth/microsoft".format(SkypeConnection.API_LOGIN),
+                                          params={"client_id": "578134", "redirect_uri": "https://web.skype.com"}))
         # This is inside some embedded JavaScript, so can't easily parse with BeautifulSoup.
-        ppftReg = re.search(r"""<input.*?name="PPFT".*?value="(.*?)""" + "\"", loginResp.text)
+        ppftReg = re.search(r"""<input.*?name="PPFT".*?value="(.*?)""" + "\"", stage1Resp.text)
         if not ppftReg:
-            raise SkypeApiException("Couldn't retrieve PPFT from login form", loginResp)
-        if "MSPRequ" not in loginResp.cookies or "MSPOK" not in loginResp.cookies:
-            raise SkypeApiException("Couldn't retrieve MSPRequ/MSPOK cookies", loginResp)
-        return {"MSPRequ": loginResp.cookies.get("MSPRequ"),
-                "MSPOK": loginResp.cookies.get("MSPOK"),
-                "PPFT": ppftReg.group(1)}
-
-    def sendCreds(self, user, pwd, params):
-        # Now pass the login credentials over.
-        loginResp = self.conn("POST", "{0}/ppsecure/post.srf".format(SkypeConnection.API_MSACC),
-                              params={"wa": "wsignin1.0", "wp": "MBI_SSL",
-                                      "wreply": "https://lw.skype.com/login/oauth/proxy?client_id=578134&site_name="
-                                                "lw.skype.com&redirect_uri=https%3A%2F%2Fweb.skype.com%2F"},
-                              cookies={"MSPRequ": params["MSPRequ"],
-                                       "MSPOK": params["MSPOK"],
-                                       "CkTst": str(int(time.time() * 1000))},
-                              data={"login": user, "passwd": pwd, "PPFT": params["PPFT"]})
-        tField = BeautifulSoup(loginResp.text, "html.parser").find(id="t")
-        if tField is None:
-            err = re.search(r"sErrTxt:'([^'\\]*(\\.[^'\\]*)*)'", loginResp.text)
-            errMsg = "Couldn't retrieve t field from login response"
-            if err:
-                errMsg = re.sub(r"<.*?>", "", err.group(1)).replace("\\'", "'").replace("\\\\", "\\")
-            raise SkypeAuthException(errMsg, loginResp)
-        return tField.get("value")
+            raise SkypeApiException("Couldn't retrieve PPFT from login form", stage1Resp)
+        ppft = ppftReg.group(1)
+        if "MSPRequ" not in stage1Resp.cookies or "MSPOK" not in stage1Resp.cookies:
+            raise SkypeApiException("Couldn't retrieve MSPRequ/MSPOK cookies", stage1Resp)
+        # Prepare the Live login page request parameters.
+        params = {"wa": "wsignin1.0", "wp": "MBI_SSL",
+                  "wreply": "https://lw.skype.com/login/oauth/proxy?client_id=578134&site_name="
+                            "lw.skype.com&redirect_uri=https%3A%2F%2Fweb.skype.com%2F"}
+        cookies = {"MSPRequ": stage1Resp.cookies.get("MSPRequ"), "MSPOK": stage1Resp.cookies.get("MSPOK")}
+        # Stage 2: Submit the user's credentials.
+        stage2Resp = self.check(self.conn("POST", "{0}/ppsecure/post.srf".format(SkypeConnection.API_MSACC),
+                                          params=params,
+                                          cookies=dict(cookies, CkTst="G{0}".format(int(time.time() * 1000))),
+                                          data={"login": user, "passwd": pwd, "PPFT": ppft, "loginoptions": "3"}))
+        opidReg = re.search(r"""opid=([A-Z0-9]+)""", stage2Resp.text, re.I)
+        if not opidReg:
+            raise SkypeApiException("Couldn't retrieve opid field from login response", stage2Resp)
+        # Stage 3: Repeat with the 'opid' parameter.
+        stage3Resp = self.check(self.conn("POST", "{0}/ppsecure/post.srf".format(SkypeConnection.API_MSACC),
+                                          params=params,
+                                          cookies=dict(cookies, CkTst="G{0}".format(int(time.time() * 1000))),
+                                          data={"opid": opidReg.group(1), "PPFT": ppft, "site_name": "lw.skype.com",
+                                                "oauthPartner": "999", "client_id": "578134",
+                                                "redirect_uri": "https://web.skype.com", "type": "28"}))
+        # No check matches, and no further actions we can take.
+        raise SkypeApiException("Couldn't retrieve t field from login response", stage3Resp)
 
     def getToken(self, t):
         # Now exchange the 't' value for a Skype token.
